import get from "lodash/get";
import { normaliseRootArgs } from "./args";
export const readActions = ["include", "select"];
export const writeActions = [
    "create",
    "update",
    "upsert",
    "connectOrCreate",
    "createMany",
    "updateMany",
    "delete",
    "deleteMany",
];
export function isReadAction(action) {
    return readActions.includes(action);
}
export function isWriteAction(action) {
    return writeActions.includes(action);
}
const fieldsByWriteAction = {
    create: ["data"],
    update: ["data"],
    upsert: ["update", "create"],
    connectOrCreate: ["create"],
    createMany: ["data"],
    updateMany: ["data"],
    delete: [],
    deleteMany: [],
};
export function extractNestedWriteActions(params, relation) {
    if (!isWriteAction(params.action)) {
        return [];
    }
    const model = relation.type;
    const runInTransaction = params.runInTransaction;
    const nestedWriteActions = [];
    const fields = fieldsByWriteAction[params.action] || [];
    fields.forEach((field) => {
        const arg = get(params.args, [field, relation.name], {});
        Object.keys(arg)
            .filter(isWriteAction)
            .forEach((action) => {
            /*
                Add single writes passed as a list as separate operations.
      
                Checking if the operation is an array is enough since only lists of
                separate operations are passed as arrays at the top level. For example
                a nested create may be passed as an array but a nested createMany will
                pass an object with a data array.
              */
            if (Array.isArray(arg[action])) {
                nestedWriteActions.push(...arg[action].map((item, index) => ({
                    target: {
                        field,
                        relationName: relation.name,
                        action,
                        index,
                    },
                    params: {
                        model,
                        action,
                        args: normaliseRootArgs(action, item),
                        runInTransaction,
                        dataPath: [],
                        scope: params,
                        relation,
                    },
                })));
                return;
            }
            nestedWriteActions.push({
                target: {
                    field,
                    relationName: relation.name,
                    action,
                },
                params: {
                    model,
                    action,
                    args: normaliseRootArgs(action, arg[action]),
                    runInTransaction,
                    dataPath: [],
                    scope: params,
                    relation,
                },
            });
        });
    });
    return nestedWriteActions;
}
export function extractNestedReadActions(params, relation) {
    const model = relation.type;
    const runInTransaction = params.runInTransaction;
    const relationName = relation.name;
    const nestedReadActions = [];
    readActions.forEach((action) => {
        const arg = get(params.args, [action, relation.name]);
        if (!arg)
            return;
        nestedReadActions.push({
            target: { action, relationName: relation.name },
            params: {
                model,
                action,
                args: normaliseRootArgs(action, arg),
                runInTransaction,
                dataPath: [],
                scope: params,
                relation,
            },
        });
        // push select nested in an include
        if (action === "include" && arg.select) {
            nestedReadActions.push({
                target: { field: "include", action: "select", relationName },
                params: {
                    model,
                    action: "select",
                    args: normaliseRootArgs("select", arg.select),
                    runInTransaction,
                    dataPath: [],
                    scope: params,
                    relation,
                },
            });
        }
    });
    return nestedReadActions;
}
export function extractNestedActions(params, relation) {
    return [
        ...extractNestedWriteActions(params, relation),
        ...extractNestedReadActions(params, relation),
    ];
}
