"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractNestedActions = exports.extractNestedReadActions = exports.extractNestedWriteActions = exports.isWriteAction = exports.isReadAction = exports.writeActions = exports.readActions = void 0;
const get_1 = __importDefault(require("lodash/get"));
const args_1 = require("./args");
exports.readActions = ["include", "select"];
exports.writeActions = [
    "create",
    "update",
    "upsert",
    "connectOrCreate",
    "createMany",
    "updateMany",
    "delete",
    "deleteMany",
];
function isReadAction(action) {
    return exports.readActions.includes(action);
}
exports.isReadAction = isReadAction;
function isWriteAction(action) {
    return exports.writeActions.includes(action);
}
exports.isWriteAction = isWriteAction;
const fieldsByWriteAction = {
    create: ["data"],
    update: ["data"],
    upsert: ["update", "create"],
    connectOrCreate: ["create"],
    createMany: ["data"],
    updateMany: ["data"],
    delete: [],
    deleteMany: [],
};
function extractNestedWriteActions(params, relation) {
    if (!isWriteAction(params.action)) {
        return [];
    }
    const model = relation.type;
    const runInTransaction = params.runInTransaction;
    const nestedWriteActions = [];
    const fields = fieldsByWriteAction[params.action] || [];
    fields.forEach((field) => {
        const arg = (0, get_1.default)(params.args, [field, relation.name], {});
        Object.keys(arg)
            .filter(isWriteAction)
            .forEach((action) => {
            /*
                Add single writes passed as a list as separate operations.
      
                Checking if the operation is an array is enough since only lists of
                separate operations are passed as arrays at the top level. For example
                a nested create may be passed as an array but a nested createMany will
                pass an object with a data array.
              */
            if (Array.isArray(arg[action])) {
                nestedWriteActions.push(...arg[action].map((item, index) => ({
                    target: {
                        field,
                        relationName: relation.name,
                        action,
                        index,
                    },
                    params: {
                        model,
                        action,
                        args: (0, args_1.normaliseRootArgs)(action, item),
                        runInTransaction,
                        dataPath: [],
                        scope: params,
                        relation,
                    },
                })));
                return;
            }
            nestedWriteActions.push({
                target: {
                    field,
                    relationName: relation.name,
                    action,
                },
                params: {
                    model,
                    action,
                    args: (0, args_1.normaliseRootArgs)(action, arg[action]),
                    runInTransaction,
                    dataPath: [],
                    scope: params,
                    relation,
                },
            });
        });
    });
    return nestedWriteActions;
}
exports.extractNestedWriteActions = extractNestedWriteActions;
function extractNestedReadActions(params, relation) {
    const model = relation.type;
    const runInTransaction = params.runInTransaction;
    const relationName = relation.name;
    const nestedReadActions = [];
    exports.readActions.forEach((action) => {
        const arg = (0, get_1.default)(params.args, [action, relation.name]);
        if (!arg)
            return;
        nestedReadActions.push({
            target: { action, relationName: relation.name },
            params: {
                model,
                action,
                args: (0, args_1.normaliseRootArgs)(action, arg),
                runInTransaction,
                dataPath: [],
                scope: params,
                relation,
            },
        });
        // push select nested in an include
        if (action === "include" && arg.select) {
            nestedReadActions.push({
                target: { field: "include", action: "select", relationName },
                params: {
                    model,
                    action: "select",
                    args: (0, args_1.normaliseRootArgs)("select", arg.select),
                    runInTransaction,
                    dataPath: [],
                    scope: params,
                    relation,
                },
            });
        }
    });
    return nestedReadActions;
}
exports.extractNestedReadActions = extractNestedReadActions;
function extractNestedActions(params, relation) {
    return [
        ...extractNestedWriteActions(params, relation),
        ...extractNestedReadActions(params, relation),
    ];
}
exports.extractNestedActions = extractNestedActions;
