"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildParamsFromCalls = exports.assertActionChangeIsValid = void 0;
const set_1 = __importDefault(require("lodash/set"));
const get_1 = __importDefault(require("lodash/get"));
const unset_1 = __importDefault(require("lodash/unset"));
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const targets_1 = require("./targets");
const results_1 = require("./results");
const args_1 = require("./args");
const actions_1 = require("./actions");
function addWriteArgsToParams(params, target, args) {
    const targetPath = (0, targets_1.buildWriteTargetPath)(target);
    const targetArgs = (0, get_1.default)(params.args, targetPath);
    // if target doesn't exist, we can just set the updated args
    if (!targetArgs) {
        (0, set_1.default)(params.args, targetPath, args);
        return;
    }
    // createMany operations cannot be turned into arrays of operations so merge
    // their data fields
    if (target.action === "createMany") {
        (0, set_1.default)(params.args, [...targetPath, "data"], [...targetArgs.data, ...args.data]);
        return;
    }
    // if target is an array of operations push args as another operation
    if (Array.isArray(targetArgs)) {
        targetArgs.push(args);
        return;
    }
    // convert target to an array of operations with the target args as the
    // first operation and passed args as the second
    (0, set_1.default)(params.args, targetPath, [targetArgs, args]);
}
function removeWriteArgsFromParams(params, target) {
    // remove args from target
    (0, unset_1.default)(params.args, (0, targets_1.buildWriteTargetPath)(target));
    // TODO:- check if we must handle createMany operations
    // if target parent is now an empty object or array we must remove it
    const targetParentPath = [target.field, target.relationName];
    const targetParent = (0, get_1.default)(params.args, targetParentPath);
    if (Object.keys(targetParent).length === 0) {
        (0, unset_1.default)(params.args, targetParentPath);
    }
}
function removeReadArgsFromParams(params, target) {
    const targetPath = (0, targets_1.buildReadTargetPath)(target);
    // remove args from target
    (0, unset_1.default)(params.args, targetPath);
    // if target parent is an array with only unset values we must remove it
    const targetParentPath = targetPath.slice(0, -1);
    const targetParent = (0, get_1.default)(params.args, targetParentPath);
    if (Object.keys(targetParent).length === 0) {
        (0, unset_1.default)(params.args, targetParentPath);
    }
}
function assertActionChangeIsValid(previousAction, nextAction) {
    if ((0, actions_1.isReadAction)(previousAction) && (0, actions_1.isWriteAction)(nextAction)) {
        throw new Error("Changing a read action to a write action is not supported");
    }
    if ((0, actions_1.isWriteAction)(previousAction) && (0, actions_1.isReadAction)(nextAction)) {
        throw new Error("Changing a write action to a read action is not supported");
    }
}
exports.assertActionChangeIsValid = assertActionChangeIsValid;
function buildParamsFromCalls(calls, parentParams) {
    const finalParams = (0, cloneDeep_1.default)(parentParams);
    calls.forEach(({ target, origin, updatedParams }) => {
        assertActionChangeIsValid(origin.action, target.action);
        if ((0, targets_1.isWriteTarget)(target) && (0, targets_1.isWriteTarget)(origin)) {
            const targetPath = (0, targets_1.buildWriteTargetPath)(target);
            const targetArgs = (0, get_1.default)(finalParams.args, targetPath);
            const updatedArgs = (0, args_1.normaliseNestedArgs)(target.action, updatedParams.args);
            // if target hasn't changed but is an array it has been merged
            // the original target must be the first element of the array
            if (target.action === origin.action && Array.isArray(targetArgs)) {
                targetArgs[0] = updatedArgs;
                return;
            }
            // set the updated args if the target hasn't changed
            if (target.action === origin.action) {
                (0, set_1.default)(finalParams.args, targetPath, updatedArgs);
                return;
            }
            // if action has changed we add merge args with target and remove the args
            // from the origin
            addWriteArgsToParams(finalParams, target, updatedArgs);
            removeWriteArgsFromParams(finalParams, origin);
        }
        if ((0, targets_1.isReadTarget)(target) && (0, targets_1.isReadTarget)(origin)) {
            const targetPath = (0, targets_1.buildReadTargetPath)(target);
            // Do nothing if the target action has changed and already exists. Having
            // a select and include at the same level is not supported so we should
            // pass this through so Prisma can throw an error to let the user know.
            if (origin.action !== target.action &&
                typeof (0, get_1.default)(finalParams.args, targetPath) !== "undefined") {
                return;
            }
            // because includes and selects cannot be at the same level we can safely
            // set target path to be the updated args without worrying about
            // overwriting the original args
            (0, results_1.setNestedResult)(finalParams.args, targetPath, (0, args_1.normaliseNestedArgs)(target.action, updatedParams.args));
            // remove the origin args if the action has changed
            if (target.action !== origin.action) {
                removeReadArgsFromParams(finalParams, origin);
            }
        }
    });
    return finalParams;
}
exports.buildParamsFromCalls = buildParamsFromCalls;
